"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const command_1 = require("@salesforce/command");
const rimraf = require("rimraf");
const core_1 = require("@salesforce/core");
const xml2js = require("xml2js");
const util = require("util");
// tslint:disable-next-line:ordered-imports
var jsforce = require("jsforce");
var path = require("path");
const checkRetrievalStatus_1 = require("../../../../shared/checkRetrievalStatus");
const extract_1 = require("../../../../shared/extract");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "connectedapp_retrieve");
class Retrieve extends command_1.SfdxCommand {
    async run() {
        rimraf.sync("temp_sfpowerkit");
        let retrieveRequest = {
            apiVersion: "45.0"
        };
        retrieveRequest["singlePackage"] = true;
        retrieveRequest["unpackaged"] = {
            types: { name: "ConnectedApp", members: this.flags.name }
        };
        // if(!this.flags.json)
        // this.ux.logJson(retrieveRequest);
        await this.org.refreshAuth();
        const conn = this.org.getConnection();
        this.flags.apiversion =
            this.flags.apiversion || (await conn.retrieveMaxApiVersion());
        conn.metadata.pollTimeout = 60;
        let retrievedId;
        await conn.metadata.retrieve(retrieveRequest, function (error, result) {
            if (error) {
                return console.error(error);
            }
            retrievedId = result.id;
        });
        // if(!this.flags.json)
        // console.log(retrievedId);
        let metadata_retrieve_result = await checkRetrievalStatus_1.checkRetrievalStatus(conn, retrievedId, !this.flags.json);
        if (!metadata_retrieve_result.zipFile)
            throw new core_1.SfdxError("Unable to find the requested ConnectedApp");
        var zipFileName = "temp_sfpowerkit/unpackaged.zip";
        fs_extra_1.default.mkdirSync("temp_sfpowerkit");
        fs_extra_1.default.writeFileSync(zipFileName, metadata_retrieve_result.zipFile, {
            encoding: "base64"
        });
        await extract_1.extract("temp_sfpowerkit");
        let resultFile = `temp_sfpowerkit/connectedApps/${this.flags.name}.connectedApp`;
        // if(!this.flags.json)
        // this.ux.log(`Checking for file ${resultFile}`);
        // this.ux.log(path.resolve(resultFile));
        let retrieved_connectedapp;
        if (fs_extra_1.default.existsSync(path.resolve(resultFile))) {
            const parser = new xml2js.Parser({ explicitArray: false });
            const parseString = util.promisify(parser.parseString);
            retrieved_connectedapp = await parseString(fs_extra_1.default.readFileSync(path.resolve(resultFile)));
            // if(!this.flags.json)
            // this.ux.logJson(retrieved_connectedapp);
            this.ux.log(`Retrieved ConnectedApp Succesfully  with Consumer Key : ${retrieved_connectedapp.ConnectedApp.oauthConfig.consumerKey}`);
            return { connectedapp: retrieved_connectedapp.ConnectedApp };
        }
        else {
            throw new core_1.SfdxError("Unable to process");
        }
    }
}
Retrieve.description = messages.getMessage("commandDescription");
Retrieve.examples = [
    `$ sfdx sfpowerkit:org:connectedapp:retrieve -n AzurePipelines -u azlam@sfdc.com 
  Retrived AzurePipelines Consumer Key : XSD21Sd23123w21321
  `
];
// Comment this out if your command does not require an org username
Retrieve.requiresUsername = true;
Retrieve.flagsConfig = {
    name: command_1.flags.string({
        required: true,
        char: "n",
        description: messages.getMessage("nameFlagDescription")
    })
};
exports.default = Retrieve;
//# sourceMappingURL=retrieve.js.map