"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const metadataInfo_1 = require("../../../shared/metadataInfo");
const baseMetadataRetriever_1 = __importDefault(require("./baseMetadataRetriever"));
const QUERY = "Select Id, Name, NameSpacePrefix From ApexPage";
class ApexPageRetriever extends baseMetadataRetriever_1.default {
    constructor(org) {
        super(org, true);
        this.org = org;
        super.setQuery(QUERY);
    }
    static getInstance(org) {
        if (!ApexPageRetriever.instance) {
            ApexPageRetriever.instance = new ApexPageRetriever(org);
        }
        return ApexPageRetriever.instance;
    }
    async getObjects() {
        if ((this.data === undefined || this.data.length == 0) &&
            !this.dataLoaded) {
            super.setQuery(QUERY);
            let pages = await super.getObjects();
            for (let i = 0; i < pages.length; i++) {
                let page = pages[i];
                if (!lodash_1.default.isNil(page.NamespacePrefix)) {
                    page.FullName = `${page.NamespacePrefix}__${page.Name}`;
                }
                else {
                    page.FullName = page.Name;
                }
            }
            this.data = pages;
            this.dataLoaded = true;
        }
        return this.data;
    }
    async getPages() {
        return await this.getObjects();
    }
    async pageExists(page) {
        let found = false;
        //Look first in project files
        if (!lodash_1.default.isNil(metadataInfo_1.METADATA_INFO.ApexPage.components)) {
            found = metadataInfo_1.METADATA_INFO.ApexPage.components.includes(page);
        }
        if (!found) {
            //not found, check on the org
            let pages = await this.getPages();
            let foundPage = pages.find(p => {
                return p.FullName === page;
            });
            found = !lodash_1.default.isNil(foundPage);
        }
        return found;
    }
}
exports.default = ApexPageRetriever;
//# sourceMappingURL=apexPageRetriever.js.map