"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const metadataInfo_1 = require("../../../shared/metadataInfo");
const baseMetadataRetriever_1 = __importDefault(require("./baseMetadataRetriever"));
const QUERY = "Select Id, NamespacePrefix, DeveloperName, Label From CustomApplication ";
class CustomApplicationRetriever extends baseMetadataRetriever_1.default {
    constructor(org) {
        super(org, true);
        this.org = org;
        super.setQuery(QUERY);
    }
    static getInstance(org) {
        if (!CustomApplicationRetriever.instance) {
            CustomApplicationRetriever.instance = new CustomApplicationRetriever(org);
        }
        return CustomApplicationRetriever.instance;
    }
    async getObjects() {
        if ((this.data === undefined || this.data.length == 0) &&
            !this.dataLoaded) {
            super.setQuery(QUERY);
            let apps = await super.getObjects();
            for (let i = 0; i < apps.length; i++) {
                let app = apps[i];
                if (!lodash_1.default.isNil(app.NamespacePrefix)) {
                    app.FullName = `${app.NamespacePrefix}__${app.DeveloperName}`;
                }
                else {
                    app.FullName = app.DeveloperName;
                }
            }
            this.data = apps;
            this.dataLoaded = true;
        }
        return this.data;
    }
    async getApps() {
        return await this.getObjects();
    }
    async appExists(application) {
        let found = false;
        //Look first in project files
        if (!lodash_1.default.isNil(metadataInfo_1.METADATA_INFO.CustomApplication.components)) {
            found = metadataInfo_1.METADATA_INFO.CustomApplication.components.includes(application);
        }
        if (!found) {
            //not found, check on the org
            let apps = await this.getApps();
            let foundApp = apps.find(app => {
                return app.FullName === application;
            });
            found = !lodash_1.default.isNil(foundApp);
        }
        return found;
    }
}
exports.default = CustomApplicationRetriever;
//# sourceMappingURL=customApplicationRetriever.js.map