"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const metadataInfo_1 = require("../../../shared/metadataInfo");
const lodash_1 = __importDefault(require("lodash"));
const baseMetadataRetriever_1 = __importDefault(require("./baseMetadataRetriever"));
const entityDefinitionRetriever_1 = __importDefault(require("./entityDefinitionRetriever"));
const QUERY = "SELECT Id, Name, EntityDefinitionId, NamespacePrefix From Layout ";
class LayoutRetriever extends baseMetadataRetriever_1.default {
    constructor(org) {
        super(org, true);
        this.org = org;
        super.setQuery(QUERY);
    }
    static getInstance(org) {
        if (!LayoutRetriever.instance) {
            LayoutRetriever.instance = new LayoutRetriever(org);
        }
        return LayoutRetriever.instance;
    }
    async getObjects() {
        if ((this.data === undefined || this.data.length == 0) &&
            !this.dataLoaded) {
            let entityDefinitionRetriever = entityDefinitionRetriever_1.default.getInstance(this.org);
            super.setQuery(QUERY);
            let layouts = await super.getObjects();
            for (let i = 0; i < layouts.length; i++) {
                layouts[i].ObjectName = await entityDefinitionRetriever.getObjectNameByDurableId(layouts[i].EntityDefinitionId);
                let namespace = "";
                if (layouts[i].NamespacePrefix !== undefined &&
                    layouts[i].NamespacePrefix !== "" &&
                    layouts[i].NamespacePrefix !== null &&
                    layouts[i].NamespacePrefix !== "null") {
                    namespace = layouts[i].NamespacePrefix + "__";
                }
                if (layouts[i].ObjectName !== null && layouts[i].ObjectName !== "") {
                    layouts[i].FullName =
                        layouts[i].ObjectName +
                            "-" +
                            namespace +
                            layouts[i].Name.replace("/", "%2F");
                }
                else {
                    layouts[i].FullName =
                        layouts[i].EntityDefinitionId +
                            "-" +
                            namespace +
                            layouts[i].Name.replace("/", "%2F");
                }
            }
            this.data = layouts;
            this.dataLoaded = true;
        }
        return this.data;
    }
    async getLayouts() {
        return await this.getObjects();
    }
    async layoutExists(layout) {
        let found = false;
        //Look first in project files
        if (!lodash_1.default.isNil(metadataInfo_1.METADATA_INFO.Layout.components)) {
            found = metadataInfo_1.METADATA_INFO.Layout.components.includes(layout);
        }
        if (!found) {
            //not found, check on the org
            let layouts = await this.getLayouts();
            let foundLayout = layouts.find(l => {
                return l.FullName === layout;
            });
            found = !lodash_1.default.isNil(foundLayout);
        }
        return found;
    }
}
exports.default = LayoutRetriever;
//# sourceMappingURL=layoutRetriever.js.map