"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const metadataInfo_1 = require("../../../shared/metadataInfo");
const lodash_1 = __importDefault(require("lodash"));
const baseMetadataRetriever_1 = __importDefault(require("./baseMetadataRetriever"));
const QUERY = "SELECT Id,  Name, SobjectName, DurableId, IsCustom, Label FROM TabDefinition ";
class TabDefinitionRetriever extends baseMetadataRetriever_1.default {
    constructor(org) {
        super(org, true);
        this.org = org;
        super.setQuery(QUERY);
    }
    static getInstance(org) {
        if (!TabDefinitionRetriever.instance) {
            TabDefinitionRetriever.instance = new TabDefinitionRetriever(org);
        }
        return TabDefinitionRetriever.instance;
    }
    async getObjects() {
        if ((this.data === undefined || this.data.length == 0) &&
            !this.dataLoaded) {
            super.setQuery(QUERY);
            let tabs = await super.getObjects();
            this.data = tabs;
            this.dataLoaded = true;
        }
        return this.data;
    }
    async getTabs() {
        return await this.getObjects();
    }
    async tabExists(tab) {
        let found = false;
        //Look first in project files
        if (!lodash_1.default.isNil(metadataInfo_1.METADATA_INFO.CustomTab.components)) {
            found = metadataInfo_1.METADATA_INFO.CustomTab.components.includes(tab);
        }
        if (!found) {
            //not found, check on the org
            let tabs = await this.getTabs();
            let foundTab = tabs.find(t => {
                return t.Name === tab;
            });
            found = !lodash_1.default.isNil(foundTab);
        }
        return found;
    }
}
exports.default = TabDefinitionRetriever;
//# sourceMappingURL=tabDefinitionRetriever.js.map