"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const sfpowerkit_1 = require("../../../sfpowerkit");
const metadataFiles_1 = __importDefault(require("../../../shared/metadataFiles"));
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const xml2js = require("xml2js");
const metadataInfo_1 = require("../../../shared/metadataInfo");
const profileRetriever_1 = __importDefault(require("../../metadata/retriever/profileRetriever"));
const customApplicationRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/customApplicationRetriever"));
const apexClassRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/apexClassRetriever"));
const fieldRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/fieldRetriever"));
const layoutRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/layoutRetriever"));
const recordTypeRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/recordTypeRetriever"));
const entityDefinitionRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/entityDefinitionRetriever"));
const apexPageRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/apexPageRetriever"));
const tabDefinitionRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/tabDefinitionRetriever"));
const userLicenseRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/userLicenseRetriever"));
const userPermissionBuilder_1 = __importDefault(require("../../../impl/metadata/builder/userPermissionBuilder"));
const util = require("util");
const lodash_1 = __importDefault(require("lodash"));
const profileActions_1 = __importDefault(require("./profileActions"));
const nonArayProperties = [
    "custom",
    "description",
    "fullName",
    "userLicense",
    "$"
];
class ProfileReconcile extends profileActions_1.default {
    async reconcile(srcFolders, profileList) {
        let result = [];
        this.metadataFiles = new metadataFiles_1.default();
        srcFolders.forEach(srcFolder => {
            let normalizedPath = path.join(process.cwd(), srcFolder);
            this.metadataFiles.loadComponents(normalizedPath);
        });
        profileList = profileList.map(element => {
            return element + metadataInfo_1.METADATA_INFO.Profile.sourceExtension;
        });
        await this.profileRetriever.loadSupportedPermissions();
        for (let count = 0; count < metadataInfo_1.METADATA_INFO.Profile.files.length; count++) {
            let profileComponent = metadataInfo_1.METADATA_INFO.Profile.files[count];
            if (profileList.length == 0 ||
                profileList.includes(path.basename(profileComponent))) {
                sfpowerkit_1.SfPowerKit.ux.log("Reconciling profile " + path.basename(profileComponent));
                let profileXmlString = fs.readFileSync(profileComponent);
                const parser = new xml2js.Parser({ explicitArray: true });
                const parseString = util.promisify(parser.parseString);
                let parseResult = await parseString(profileXmlString);
                let profileObj = profileRetriever_1.default.toProfile(parseResult.Profile); // as Profile
                profileObj = await this.removePermissions(profileObj);
                //Manage licences
                let licenceUtils = userLicenseRetriever_1.default.getInstance(this.org);
                const isSupportedLicence = await licenceUtils.userLicenseExists(profileObj.userLicense);
                if (!isSupportedLicence) {
                    delete profileObj.userLicense;
                }
                // remove unsupported userPermission
                let unsupportedLicencePermissions = this.profileRetriever.getUnsupportedLicencePermissions(profileObj.userLicense);
                if (profileObj.userPermissions != null &&
                    profileObj.userPermissions.length > 0) {
                    profileObj.userPermissions = profileObj.userPermissions.filter(permission => {
                        let supported = !unsupportedLicencePermissions.includes(permission.name);
                        return supported;
                    });
                }
                if (profileObj.userPermissions !== undefined &&
                    profileObj.userPermissions.length > 0) {
                    //Remove permission that are not present in the target org
                    profileObj.userPermissions = profileObj.userPermissions.filter(permission => {
                        let supported = this.profileRetriever.supportedPermissions.includes(permission.name);
                        return supported;
                    });
                }
                //UserPermissionUtils.addPermissionDependencies(profileObj);
                let isCustom = "" + profileObj.custom;
                if (isCustom == "false") {
                    delete profileObj.userPermissions;
                }
                //this.handleViewAllDataPermission(profileObj);
                //this.handleInstallPackagingPermission(profileObj);
                //this.handleQueryAllFilesPermission(profileObj);
                userPermissionBuilder_1.default.handlePermissionDependency(profileObj, this.profileRetriever.supportedPermissions);
                //Delete eampty arrays
                for (let key in profileObj) {
                    if (Array.isArray(profileObj[key])) {
                        //All top element must be arays exept non arrayProperties
                        if (!nonArayProperties.includes(key) &&
                            profileObj[key].length === 0) {
                            delete profileObj[key];
                        }
                    }
                }
                let builder = new xml2js.Builder({ rootName: "Profile" });
                let xml = builder.buildObject(profileObj);
                fs.writeFileSync(profileComponent, xml);
                result.push(profileComponent);
            }
        }
        return result;
    }
    async reconcileApp(profileObj) {
        let utils = customApplicationRetriever_1.default.getInstance(this.org);
        if (profileObj.applicationVisibilities !== undefined) {
            let validArray = [];
            for (let i = 0; i < profileObj.applicationVisibilities.length; i++) {
                let cmpObj = profileObj.applicationVisibilities[i];
                let exist = utils.appExists(cmpObj.application);
                if (exist) {
                    validArray.push(cmpObj);
                }
            }
            profileObj.applicationVisibilities = validArray;
        }
        return profileObj;
    }
    async reconcileClasses(profileObj) {
        let utils = apexClassRetriever_1.default.getInstance(this.org);
        if (profileObj.classAccesses !== undefined) {
            if (!Array.isArray(profileObj.classAccesses)) {
                profileObj.classAccesses = [profileObj.classAccesses];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.classAccesses.length; i++) {
                let cmpObj = profileObj.classAccesses[i];
                let exists = await utils.classExists(cmpObj.apexClass);
                if (exists) {
                    validArray.push(cmpObj);
                }
            }
            profileObj.classAccesses = validArray;
        }
        return profileObj;
    }
    async reconcileFields(profileObj) {
        let utils = fieldRetriever_1.default.getInstance(this.org);
        if (profileObj.fieldLevelSecurities !== undefined) {
            if (!Array.isArray(profileObj.fieldLevelSecurities)) {
                profileObj.fieldLevelSecurities = [profileObj.fieldLevelSecurities];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.fieldLevelSecurities.length; i++) {
                let cmpObj = profileObj.fieldLevelSecurities[i];
                let exists = await utils.fieldExist(cmpObj.field);
                if (exists) {
                    validArray.push(cmpObj);
                }
            }
            profileObj.fieldLevelSecurities = validArray;
        }
        if (profileObj.fieldPermissions !== undefined) {
            if (!Array.isArray(profileObj.fieldPermissions)) {
                profileObj.fieldPermissions = [profileObj.fieldPermissions];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.fieldPermissions.length; i++) {
                let cmpObj = profileObj.fieldPermissions[i];
                let exists = await utils.fieldExist(cmpObj.field);
                if (exists) {
                    validArray.push(cmpObj);
                }
            }
            profileObj.fieldPermissions = validArray;
        }
        return profileObj;
    }
    async reconcileLayouts(profileObj) {
        let utils = layoutRetriever_1.default.getInstance(this.org);
        let rtUtils = recordTypeRetriever_1.default.getInstance(this.org);
        if (profileObj.layoutAssignments !== undefined) {
            var validArray = [];
            for (let count = 0; count < profileObj.layoutAssignments.length; count++) {
                let cmpObj = profileObj.layoutAssignments[count];
                let exist = (await utils.layoutExists(cmpObj.layout)) &&
                    (lodash_1.default.isNil(cmpObj.recordType) ||
                        (await rtUtils.recordTypeExists(cmpObj.recordType)));
                if (exist) {
                    validArray.push(cmpObj);
                }
            }
            profileObj.layoutAssignments = validArray;
        }
        return profileObj;
    }
    async reconcileObjects(profileObj) {
        let utils = entityDefinitionRetriever_1.default.getInstance(this.org);
        if (profileObj.objectPermissions !== undefined) {
            if (!Array.isArray(profileObj.objectPermissions)) {
                profileObj.objectPermissions = [profileObj.objectPermissions];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.objectPermissions.length; i++) {
                let cmpObj = profileObj.objectPermissions[i];
                let exist = await utils.existObjectPermission(cmpObj.object);
                if (exist) {
                    validArray.push(cmpObj);
                }
            }
            profileObj.objectPermissions = validArray;
        }
        return profileObj;
    }
    async reconcilePages(profileObj) {
        let utils = apexPageRetriever_1.default.getInstance(this.org);
        if (profileObj.pageAccesses !== undefined) {
            if (!Array.isArray(profileObj.pageAccesses)) {
                profileObj.pageAccesses = [profileObj.pageAccesses];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.pageAccesses.length; i++) {
                let cmpObj = profileObj.pageAccesses[i];
                let exist = await utils.pageExists(cmpObj.apexPage);
                if (exist) {
                    validArray.push(cmpObj);
                }
            }
            profileObj.pageAccesses = validArray;
        }
        return profileObj;
    }
    async reconcileRecordTypes(profileObj) {
        let utils = recordTypeRetriever_1.default.getInstance(this.org);
        if (profileObj.recordTypeVisibilities !== undefined) {
            if (!Array.isArray(profileObj.recordTypeVisibilities)) {
                profileObj.recordTypeVisibilities = [profileObj.recordTypeVisibilities];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.recordTypeVisibilities.length; i++) {
                let cmpObj = profileObj.recordTypeVisibilities[i];
                let exist = await utils.recordTypeExists(cmpObj.recordType);
                if (exist) {
                    validArray.push(cmpObj);
                }
            }
            profileObj.recordTypeVisibilities = validArray;
        }
        return profileObj;
    }
    async reconcileTabs(profileObj) {
        let utils = tabDefinitionRetriever_1.default.getInstance(this.org);
        if (profileObj.tabVisibilities !== undefined) {
            if (!Array.isArray(profileObj.tabVisibilities)) {
                profileObj.tabVisibilities = [profileObj.tabVisibilities];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.tabVisibilities.length; i++) {
                let cmpObj = profileObj.tabVisibilities[i];
                let exist = await utils.tabExists(cmpObj.tab);
                if (exist) {
                    validArray.push(cmpObj);
                }
            }
            profileObj.tabVisibilities = validArray;
        }
        return profileObj;
    }
    async removePermissions(profileObj) {
        profileObj = await this.reconcileApp(profileObj);
        profileObj = await this.reconcileClasses(profileObj);
        profileObj = await this.reconcileFields(profileObj);
        profileObj = await this.reconcileObjects(profileObj);
        profileObj = await this.reconcilePages(profileObj);
        profileObj = await this.reconcileLayouts(profileObj);
        profileObj = await this.reconcileRecordTypes(profileObj);
        profileObj = await this.reconcileTabs(profileObj);
        return profileObj;
    }
}
exports.default = ProfileReconcile;
//# sourceMappingURL=profileReconcile.js.map