"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const metadataInfo_1 = require("./metadataInfo");
const fileutils_1 = __importDefault(require("./fileutils"));
const lodash_1 = __importDefault(require("lodash"));
class MetadataFiles {
    static getFullApiName(fileName) {
        let fullName = "";
        let metadateType = metadataInfo_1.MetadataInfoUtils.getMetadataName(fileName);
        let splitFilepath = fileName.split(path.sep);
        let isObjectChild = metadataInfo_1.METADATA_INFO.CustomObject.childXmlNames.includes(metadateType);
        if (isObjectChild) {
            let objectName = splitFilepath[splitFilepath.length - 3];
            let fieldName = splitFilepath[splitFilepath.length - 1].split(".")[0];
            fullName = objectName.concat("." + fieldName);
        }
        else {
            fullName = splitFilepath[splitFilepath.length - 1].split(".")[0];
        }
        return fullName;
    }
    static getFullApiNameWithExtension(fileName) {
        let fullName = "";
        let metadateType = metadataInfo_1.MetadataInfoUtils.getMetadataName(fileName);
        let splitFilepath = fileName.split(path.sep);
        let isObjectChild = metadataInfo_1.METADATA_INFO.CustomObject.childXmlNames.includes(metadateType);
        if (isObjectChild) {
            let objectName = splitFilepath[splitFilepath.length - 3];
            let fieldName = splitFilepath[splitFilepath.length - 1];
            fullName = objectName.concat("." + fieldName);
        }
        else {
            fullName = splitFilepath[splitFilepath.length - 1];
        }
        return fullName;
    }
    static isCustomMetadata(filepath, name) {
        let result = true;
        let splitFilepath = filepath.split(path.sep);
        let componentName = splitFilepath[splitFilepath.length - 1];
        componentName = componentName.substring(0, componentName.indexOf("."));
        if (name === metadataInfo_1.METADATA_INFO.CustomField.xmlName || name === metadataInfo_1.METADATA_INFO.CustomObject.xmlName) {
            //Custom Field or Custom Object
            result = componentName.endsWith("__c") || componentName.endsWith("__mdt");
        }
        return result;
    }
    static getMemberNameFromFilepath(filepath, name) {
        let member;
        let splitFilepath = filepath.split(path.sep);
        let lastIndex = splitFilepath.length - 1;
        let isObjectChild = metadataInfo_1.METADATA_INFO.CustomObject.childXmlNames.includes(name);
        let metadataDescribe = metadataInfo_1.METADATA_INFO[name];
        if (isObjectChild) {
            let objectName = splitFilepath[lastIndex - 2];
            let fieldName = splitFilepath[lastIndex].split(".")[0];
            member = objectName.concat("." + fieldName);
        }
        else if (metadataDescribe.inFolder) {
            let baseName = metadataDescribe.directoryName;
            let baseIndex = filepath.indexOf(baseName) + baseName.length;
            let cmpPath = filepath.substring(baseIndex + 1); // add 1 to remove the path seperator
            cmpPath = cmpPath.substring(0, cmpPath.indexOf("."));
            member = cmpPath.replace(path.sep, "/");
        }
        else {
            member = splitFilepath[lastIndex].replace(metadataInfo_1.SOURCE_EXTENSION_REGEX, "");
        }
        return member;
    }
    loadComponents(srcFolder) {
        var metadataFiles = fileutils_1.default.getAllFilesSync(srcFolder);
        let keys = Object.keys(metadataInfo_1.METADATA_INFO);
        if (Array.isArray(metadataFiles) && metadataFiles.length > 0) {
            metadataFiles.forEach(metadataFile => {
                for (let i = 0; i < keys.length; i++) {
                    let match = false;
                    if (metadataFile.endsWith(metadataInfo_1.METADATA_INFO[keys[i]].sourceExtension)) {
                        match = true;
                    }
                    else if (metadataInfo_1.METADATA_INFO[keys[i]].inFolder &&
                        metadataFile.endsWith(metadataInfo_1.METADATA_INFO[keys[i]].folderExtension)) {
                        match = true;
                    }
                    if (match) {
                        if (lodash_1.default.isNil(metadataInfo_1.METADATA_INFO[keys[i]].files)) {
                            metadataInfo_1.METADATA_INFO[keys[i]].files = [];
                            metadataInfo_1.METADATA_INFO[keys[i]].components = [];
                        }
                        metadataInfo_1.METADATA_INFO[keys[i]].files.push(metadataFile);
                        let name = fileutils_1.default.getFileNameWithoutExtension(metadataFile, metadataInfo_1.METADATA_INFO[keys[i]].sourceExtension);
                        if (metadataInfo_1.METADATA_INFO[keys[i]].isChildComponent) {
                            let fileParts = metadataFile.split(path.sep);
                            let parentName = fileParts[fileParts.length - 3];
                            name = parentName + "." + name;
                        }
                        metadataInfo_1.METADATA_INFO[keys[i]].components.push(name);
                        break;
                    }
                }
            });
        }
        else {
            keys.forEach(key => {
                if (lodash_1.default.isNil(metadataInfo_1.METADATA_INFO[key].files)) {
                    metadataInfo_1.METADATA_INFO[key].files = [];
                    metadataInfo_1.METADATA_INFO[key].components = [];
                }
            });
        }
    }
}
exports.default = MetadataFiles;
//# sourceMappingURL=metadataFiles.js.map