"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = __importStar(require("lodash"));
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
exports.SOURCE_EXTENSION_REGEX = /\.[a-zA-Z]+-meta\.xml/;
const SPLITED_TYPES = {
    CustomField: {
        suffix: "field",
        folder: "fields"
    },
    BusinessProcess: {
        suffix: "businessProcess",
        folder: "businessProcesses"
    },
    CompactLayout: {
        suffix: "compactLayout",
        folder: "compactLayouts"
    },
    FieldSet: {
        suffix: "fieldSet",
        folder: "fieldSets"
    },
    RecordType: {
        suffix: "recordType",
        folder: "recordTypes"
    },
    ListView: {
        suffix: "listView",
        folder: "listViews"
    },
    SharingReason: {
        suffix: "sharingReason",
        folder: "sharingReasons"
    },
    ValidationRule: {
        suffix: "validationRule",
        folder: "validationRules"
    },
    WebLink: {
        suffix: "webLink",
        folder: "webLinks"
    }
};
class MetadataInfoUtils {
    static loadMetadataInfo() {
        let metadataInfo = {};
        let resourcePAth = path.join(__dirname, "..", "..", "resources", "metadatainfo.json");
        const fileData = fs.readFileSync(resourcePAth, "utf8");
        let metadataInfoJSON = JSON.parse(fileData);
        metadataInfoJSON.metadataObjects.forEach(metadata => {
            let metadataDescribe = metadata;
            if (_.isNil(metadata.suffix)) {
                if (metadata.xmlName === "AuraDefinitionBundle") {
                    metadata.suffix = "cmp";
                    metadataDescribe.suffix = "cmp";
                }
                else if (metadata.xmlName == "LightningComponentBundle") {
                    metadata.suffix = "js";
                    metadataDescribe.suffix = "js";
                }
            }
            metadataDescribe.sourceExtension = `.${metadata.suffix}-meta.xml`;
            if (metadata.inFolder) {
                let folderExtensionPrefix = metadata.suffix;
                if (_.isNil(metadata.suffix)) {
                    folderExtensionPrefix =
                        metadata.xmlName.charAt(0).toLowerCase + metadata.xmlName.slice(1);
                }
                metadataDescribe.folderExtension = `.${folderExtensionPrefix}Folder-meta.xml`;
            }
            //Generate Describe of cheildItems if exists
            if (!_.isNil(metadata.childXmlNames)) {
                metadata.childXmlNames.forEach(element => {
                    let splitedElement = SPLITED_TYPES[element];
                    if (!_.isNil(splitedElement)) {
                        let childDescribe = {};
                        childDescribe.directoryName = SPLITED_TYPES[element].folder;
                        childDescribe.suffix = SPLITED_TYPES[element].suffix;
                        childDescribe.xmlName = element;
                        childDescribe.inFolder = false;
                        childDescribe.metaFile = false;
                        childDescribe.isChildComponent = true;
                        childDescribe.sourceExtension = `.${SPLITED_TYPES[element].suffix}-meta.xml`;
                        metadataInfo[childDescribe.xmlName] = childDescribe;
                    }
                });
            }
            metadataInfo[metadataDescribe.xmlName] = metadataDescribe;
        });
        return metadataInfo;
    }
    static getMetadataName(metadataFile) {
        let matcher = metadataFile.match(exports.SOURCE_EXTENSION_REGEX);
        let extension = "";
        if (matcher) {
            extension = matcher[0];
        }
        else {
            extension = path.parse(metadataFile).ext;
        }
        //SfPowerKit.ux.log(extension);
        let metadataName = "";
        let keys = Object.keys(exports.METADATA_INFO);
        for (let i = 0; i < keys.length; i++) {
            let metaDescribe = exports.METADATA_INFO[keys[i]];
            if (metaDescribe.sourceExtension === extension ||
                metaDescribe.folderExtension === extension) {
                metadataName = metaDescribe.xmlName;
                break;
            }
        }
        return metadataName;
    }
}
exports.MetadataInfoUtils = MetadataInfoUtils;
exports.METADATA_INFO = MetadataInfoUtils.loadMetadataInfo();
//# sourceMappingURL=metadataInfo.js.map